﻿using CustomComponents.Exceptions;
using CustomComponents.Helpers;
using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Reflection;
using System.Text;
using System.Threading.Tasks;
using System.Windows.Forms;

namespace KOP_Labs
{
    public partial class CustomTree : UserControl
    {
        private List<string> hierarchy;

        public CustomTree()
        {
            InitializeComponent();
            hierarchy = new List<string>();
        }

        public void SetHierarcy(List<string> hierarchy)
        {
            this.hierarchy = hierarchy;
        }
        public int SelectedNodeIndex
        {
            get
            {
                return treeView.SelectedNode?.Index ?? -1;
            }
            set
            {
                if (value >= 0 && value < treeView.Nodes.Count)
                {
                    treeView.SelectedNode = treeView.Nodes[value];
                }
            }
        }

        public void AddObjectToTree<T>(T obj)
        {
            if (hierarchy == null)
            {
                throw new HierarcyNullException("Иерархия не задана");
            }
            TreeNode currentNode = treeView.Nodes.Count > 0 ? treeView.Nodes[0] : treeView.Nodes.Add("");
            foreach (var property in hierarchy)
            {
                var value = obj.GetType().GetProperty(property).GetValue(obj, null).ToString();

                var childNode = currentNode.Nodes.Cast<TreeNode>().FirstOrDefault(n => n.Text == value);
                if (childNode == null)
                {
                    childNode = currentNode.Nodes.Add(value);
                }

                currentNode = childNode;

            }
        }    

        public T GetSelectedObject<T>() where T : new()
        {
            if (hierarchy == null)
            {
                throw new HierarcyNullException("Иерархия не задана");
            }
            if (treeView.SelectedNode == null)
            {
                throw new InvalidOperationException("Узел не выбран");
            }

            var node = treeView.SelectedNode;
            if (node.Nodes.Count != 0)
            {
                throw new InvalidOperationException("Узел не конечный, сформировать объект нельзя");
            }

            var obj = new T();
            foreach (var property in hierarchy)
            {
                var value = node.Text;
                var propInfo = typeof(T).GetProperty(property);
                if (propInfo == null)
                {
                    throw new InvalidOperationException($"Свойство {property} не найдено в классе {typeof(T).Name}");
                }
                propInfo.SetValue(obj, Convert.ChangeType(value, propInfo.PropertyType));

                node = node.Parent;
                if (node == null)
                {
                    break;
                }
            }
            return obj;
        }
    }
}
